@@ -1,3 +1,8 @@
+import logging
+
+logger = logging.getLogger(__name__)
+
+
 def resolve_dotted_attr(instance, dotted_path):
     """
     Recursively resolve a dotted attribute path, e.g., "type.category".
@@ -68,9 +73,15 @@ class HasChanged(HookCondition):
     def check(self, instance, original_instance=None):
         if not original_instance:
             return False
+        
         current = resolve_dotted_attr(instance, self.field)
         previous = resolve_dotted_attr(original_instance, self.field)
-        return (current != previous) == self.has_changed
+        
+        result = (current != previous) == self.has_changed
+        # Only log when there's an actual change to reduce noise
+        if result:
+            logger.debug(f"HasChanged {self.field} detected change on instance {getattr(instance, 'pk', 'No PK')}")
+        return result
 
 
 class WasEqual(HookCondition):
